@@ -14,7 +14,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * $Id: cgsi_plugin.c 6800 2014-06-13 08:55:50Z msalicho $
+ * $Id: cgsi_plugin.c 7066 2014-10-28 08:54:29Z aalvarez $
  */
 
 /** cgsi_plugin.c - GSI plugin for gSOAP
@@ -134,6 +134,8 @@ int server_cgsi_plugin(struct soap *soap, struct soap_plugin *p, void *arg)
     p->fdelete = cgsi_plugin_delete;
     if (p->data)
         {
+            ((struct cgsi_plugin_data*)p->data)->start_new_line = 1;
+
             if (server_cgsi_plugin_init(soap, (struct cgsi_plugin_data*)p->data) ||
                     cgsi_parse_opts((struct cgsi_plugin_data *)p->data, arg,0))
                 {
@@ -142,7 +144,6 @@ int server_cgsi_plugin(struct soap *soap, struct soap_plugin *p, void *arg)
                     return SOAP_EOM; /* return error */
                 }
         }
-
     return SOAP_OK;
 }
 
@@ -632,7 +633,7 @@ static int server_cgsi_plugin_accept(struct soap *soap)
         {
             if (retrieve_userca_and_voms_creds(soap))
                 {
-                    cgsi_err(soap, "Error retrieveing the userca/VOMS credentials");
+                    cgsi_err(soap, "Error retrieving the userca/VOMS credentials");
                     goto error;
                 }
         }
@@ -802,6 +803,8 @@ int client_cgsi_plugin(struct soap *soap, struct soap_plugin *p, void *arg)
     p->fdelete = cgsi_plugin_delete;
     if (p->data)
         {
+            ((struct cgsi_plugin_data*)p->data)->start_new_line = 1;
+
             if (client_cgsi_plugin_init(soap, (struct cgsi_plugin_data*)p->data) ||
                     cgsi_parse_opts((struct cgsi_plugin_data *)p->data, arg,1))
                 {
@@ -850,6 +853,7 @@ static int client_cgsi_plugin_import_cred(struct soap *soap,
     size_t cert_size = 0;
     size_t key_size = 0;
     FILE* fd = NULL;
+    int key_is_cert = 0;
 
     buffer.value = NULL;
     buffer.length = 0;
@@ -864,8 +868,11 @@ static int client_cgsi_plugin_import_cred(struct soap *soap,
     cert_size = st.st_size;
 
     if (data->x509_key)
+        key_is_cert = strcmp(data->x509_cert, data->x509_key) == 0;
+
+    if (data->x509_key && !key_is_cert)
         {
-            if (stat(data->x509_key, &st) == 0)
+            if (stat(data->x509_key, &st) != 0)
                 {
                     strerror_r(errno, err_buffer, sizeof(err_buffer));
                     cgsi_err(soap, err_buffer);
@@ -893,7 +900,7 @@ static int client_cgsi_plugin_import_cred(struct soap *soap,
     fread(buffer.value, cert_size, 1, fd);
     fclose(fd);
 
-    if (data->x509_key && strcmp(data->x509_key, data->x509_cert) != 0)
+    if (data->x509_key && !key_is_cert)
         {
             fd = fopen(data->x509_key, "r");
             if (!fd)
@@ -958,14 +965,18 @@ static int client_cgsi_plugin_open(struct soap *soap,
     /* Getting the credenttials */
     if (data->x509_cert)
         {
-            trace(data, "Using gss_import_cred to load credentials");
+            trace(data, "Using gss_import_cred to load credentials\n");
             // client_cgsi_plugin_import_cred should set the error itself
-            if (client_cgsi_plugin_import_cred(soap, data) != 0)
+            if (client_cgsi_plugin_import_cred(soap, data) != 0) {
+                char buf[TBUFSIZE];
+                snprintf(buf, TBUFSIZE, "Could NOT import client credentials from %s/%s\n", data->x509_cert, data->x509_key);
+                trace(data, buf);
                 goto error;
+            }
         }
     else
         {
-            trace(data, "Using gss_acquire_cred to load credentials");
+            trace(data, "Using gss_acquire_cred to load credentials\n");
             major_status = gss_acquire_cred(&minor_status,
                                             GSS_C_NO_NAME,
                                             0,
@@ -977,6 +988,7 @@ static int client_cgsi_plugin_open(struct soap *soap,
 
             if (major_status != GSS_S_COMPLETE)
                 {
+                    trace(data, "Could NOT load client credentials\n");
                     cgsi_gssapi_err(soap,
                                     "Could NOT load client credentials",
                                     major_status,
@@ -1034,7 +1046,8 @@ static int client_cgsi_plugin_open(struct soap *soap,
     if (data->socket_fd < 0)
         {
             char buf[BUFSIZE];
-            snprintf(buf, BUFSIZE, "could not open connection to %s", hostname);
+            snprintf(buf, BUFSIZE, "could not open connection to %s:%d\n", hostname, port);
+            trace(data, buf);
             cgsi_err(soap, buf);
             goto error;
         }
@@ -1360,7 +1373,7 @@ static int client_cgsi_plugin_close(struct soap *soap)
 
 static int cgsi_plugin_copy(struct soap *soap, struct soap_plugin *dst, struct soap_plugin *src)
 {
-    struct cgsi_plugin_data *dst_data;
+    struct cgsi_plugin_data *dst_data, *src_data;
 
     *dst = *src;
     dst->data =  (struct cgsi_plugin_data *)malloc(sizeof(struct cgsi_plugin_data));
@@ -1374,6 +1387,7 @@ static int cgsi_plugin_copy(struct soap *soap, struct soap_plugin *dst, struct s
     */
 
     dst_data = (struct cgsi_plugin_data *)dst->data;
+    src_data = (struct cgsi_plugin_data *)src->data;
 
     /* don't want to share these with the source */
     dst_data->deleg_credential_handle = GSS_C_NO_CREDENTIAL;
@@ -1383,6 +1397,11 @@ static int cgsi_plugin_copy(struct soap *soap, struct soap_plugin *dst, struct s
     dst_data->deleg_credential_token = NULL;
     dst_data->fqan = NULL;
 
+    if (src_data->x509_cert)
+        dst_data->x509_cert = strdup(src_data->x509_cert);
+    if (src_data->x509_key)
+        dst_data->x509_key = strdup(src_data->x509_key);
+
     /* reset everything else connection related */
     free_conn_state(dst_data);
 
@@ -1406,6 +1425,8 @@ static void cgsi_plugin_delete(struct soap *soap, struct soap_plugin *p)
         }
 
     free_conn_state(data);
+    free(data->x509_cert);
+    free(data->x509_key);
     free(p->data);
     p->data = NULL;
 
@@ -1761,7 +1782,7 @@ int cgsi_plugin_recv_token(void *arg, void **token, size_t *token_length)
 
     {
         char buf[TBUFSIZE];
-        snprintf(buf, TBUFSIZE,  "================= RECVING: %x\n", len + SSLHSIZE);
+        snprintf(buf, TBUFSIZE,  "================= RECVING: %d\n", len + SSLHSIZE);
         trace(data, buf);
     }
     cgsi_plugin_print_token(data, tok, len+SSLHSIZE);
@@ -1788,7 +1809,7 @@ int cgsi_plugin_send_token(void *arg, void *token, size_t token_length)
 
     {
         char buf[TBUFSIZE];
-        snprintf(buf, TBUFSIZE,  "================= SENDING: %x\n",
+        snprintf(buf, TBUFSIZE,  "================= SENDING: %d\n",
                  (unsigned int)token_length);
         trace(data, buf);
     }
@@ -1824,7 +1845,7 @@ void cgsi_plugin_print_token(struct cgsi_plugin_data *data, char *token, int len
 
     /* can avoid printing all the token if the trace routine
      * is disabled */
-    if (!data->trace_mode)
+    if (data->trace_mode < 2)
         {
             return;
         }
@@ -2114,13 +2135,16 @@ static int setup_trace(struct cgsi_plugin_data *data)
 {
     char *envar;
 
-    data->trace_mode=0;
-    data->trace_file[0]= data->trace_file[CGSI_MAXNAMELEN-1]= '\0';
+    data->trace_mode = 0;
+    data->trace_file[0] = data->trace_file[CGSI_MAXNAMELEN-1]= '\0';
 
     envar = getenv(CGSI_TRACE);
     if (envar != NULL)
         {
-            data->trace_mode=1;
+            errno = 0;
+            data->trace_mode = strtol(envar, NULL, 10);
+            if (errno)
+                data->trace_mode = 1;
             envar = getenv(CGSI_TRACEFILE);
             if (envar != NULL)
                 {
@@ -2151,7 +2175,17 @@ static int trace_str(struct cgsi_plugin_data *data, const char *msg, int len)
     /* If no trace file defined, write to stderr */
     if (data->trace_file[0]=='\0')
         {
-            fprintf(stderr, "%.*s", len, msg);
+            int i;
+            for (i = 0; i < len; ++i) {
+                if (data->start_new_line) {
+                    fputs("[CGSI-GSOAP] ", stderr);
+                    data->start_new_line = 0;
+                }
+
+                fputc(msg[i], stderr);
+                if (msg[i] == '\n')
+                    data->start_new_line = 1;
+            }
         }
     else
         {
@@ -2462,7 +2496,7 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
     data = (struct cgsi_plugin_data*)soap_lookup_plugin(soap, server_plugin_id);
     if (data == NULL)
         {
-            cgsi_err(soap, "retrieve_voms: could not get data structure");
+            cgsi_err(soap, "retrieve_userca_and_voms_creds: could not get data structure");
             return -1;
         }
 
@@ -2470,6 +2504,7 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
     /* connection initialization resets this structure  */
     if (data->fqan != NULL)
         {
+            trace(data, "retrieve_userca_and_voms_creds: data->fqans already initialized\n");
             return 0;
         }
 
@@ -2480,6 +2515,7 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
     /* cast to gss_cred_id_desc */
     if (cred == GSS_C_NO_CREDENTIAL)
         {
+            trace(data, "retrieve_userca_and_voms_creds: No credentials given\n");
             goto leave;
         }
 
@@ -2487,6 +2523,7 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
 
     if (globus_module_activate(GLOBUS_GSI_CREDENTIAL_MODULE) != GLOBUS_SUCCESS)
         {
+            trace(data, "retrieve_userca_and_voms_creds: Could not activate GLOBUS_GSI_CREDENTIAL_MODULE\n");
             goto leave;
         }
 
@@ -2494,6 +2531,7 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
     gsi_cred_handle = cred_desc->cred_handle;
     if (globus_gsi_cred_get_cert(gsi_cred_handle, &px509_cred) != GLOBUS_SUCCESS)
         {
+            trace(data, "retrieve_userca_and_voms_creds: failed to get the credentials\n");
             globus_module_deactivate(GLOBUS_GSI_CREDENTIAL_MODULE);
             goto leave;
         }
@@ -2501,13 +2539,16 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
     /* Getting the certificate chain */
     if (globus_gsi_cred_get_cert_chain (gsi_cred_handle, &px509_chain) != GLOBUS_SUCCESS)
         {
+            trace(data, "retrieve_userca_and_voms_creds: failed to get the credentials chain\n");
             X509_free (px509_cred);
             (void)globus_module_deactivate (GLOBUS_GSI_CREDENTIAL_MODULE);
             goto leave;
         }
 
-    if (_get_user_ca (px509_cred, px509_chain, data->user_ca) < 0)
+    if (_get_user_ca (px509_cred, px509_chain, data->user_ca) < 0) {
+        trace(data, "retrieve_userca_and_voms_creds: could not get the user's CA\n");
         goto leave;
+    }
 
     /* No need for the globus module anymore, the rest are calls to VOMS */
     (void)globus_module_deactivate (GLOBUS_GSI_CREDENTIAL_MODULE);
@@ -2516,11 +2557,13 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
 
     if (data->disable_voms_check)
         {
+            trace(data, "retrieve_userca_and_voms_creds: voms_check disabled\n");
             ret = 0;
             goto leave;
         }
     if ((vd = VOMS_Init (NULL, NULL)) == NULL)
         {
+            trace(data, "retrieve_userca_and_voms_creds: failed to initialize VOMS\n");
             goto leave;
         }
 
@@ -2529,6 +2572,9 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
         {
             char buffer[BUFSIZE];
             VOMS_ErrorMessage(vd, error, buffer, BUFSIZE);
+            trace(data, "retrieve_userca_and_voms_creds: failed to get the VOMS extensions\n");
+            trace(data, buffer);
+            trace(data, "\n");
             cgsi_err(soap, buffer);
             VOMS_Destroy (vd);
             goto leave;
@@ -2536,15 +2582,18 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
 
     volist = vd->data;
 
-    if (volist !=NULL)
+    if (volist != NULL)
         {
             int i = 0;
             int nbfqan;
+            char buffer[BUFSIZE];
 
             /* Copying the voname */
             if ((*volist)->voname != NULL)
                 {
                     data->voname = strdup((*volist)->voname);
+                    snprintf(buffer, BUFSIZE, "retrieve_userca_and_voms_creds: got VO %s\n", data->voname);
+                    trace(data, buffer);
                 }
 
 
@@ -2563,12 +2612,18 @@ int retrieve_userca_and_voms_creds(struct soap *soap)
                             for (i=0; i<nbfqan; i++)
                                 {
                                     data->fqan[i] = strdup( volist[0]->fqan[i]);
+                                    snprintf(buffer, BUFSIZE, "retrieve_userca_and_voms_creds: got FQAN %s\n", data->fqan[i]);
+                                    trace(data, buffer);
                                 }
                             data->fqan[nbfqan] = NULL;
                             data->nbfqan = nbfqan;
                         }
                 } /* if (nbfqan > 0) */
         }
+    else
+        {
+            trace(data, "retrieve_userca_and_voms_creds: no vos present\n");
+        }
     VOMS_Destroy (vd);
 
 #endif
